@@ -130,59 +130,53 @@ static void
 ecore_system_modules_load(void)
 {
    char buf[PATH_MAX] = "";
-   char *path;
 
-   if (getenv("EFL_RUN_IN_TREE"))
+#if defined(HAVE_GETUID) && defined(HAVE_GETEUID)
+   if (getuid() == geteuid())
+#endif
      {
-        struct stat st;
-        snprintf(buf, sizeof(buf), "%s/src/modules/ecore/system",
-                 PACKAGE_BUILD_DIR);
-        if (stat(buf, &st) == 0)
+        if (getenv("EFL_RUN_IN_TREE"))
           {
-             const char *built_modules[] = {
+             struct stat st;
+             snprintf(buf, sizeof(buf), "%s/src/modules/ecore/system",
+                      PACKAGE_BUILD_DIR);
+             if (stat(buf, &st) == 0)
+               {
+                  const char *built_modules[] = {
 #ifdef HAVE_SYSTEMD
-               "systemd",
+                     "systemd",
 #endif
 #ifdef HAVE_TIZEN_CONFIGURATION_MANAGER
-               "tizen",
+                     "tizen",
 #endif
-               NULL
-             };
-             const char **itr;
-             for (itr = built_modules; *itr != NULL; itr++)
-               {
-                  snprintf(buf, sizeof(buf),
-                           "%s/src/modules/ecore/system/%s/.libs",
-                           PACKAGE_BUILD_DIR, *itr);
-                  module_list = eina_module_list_get(module_list, buf,
-                                                     EINA_FALSE, NULL, NULL);
+                     NULL
+                  };
+                  const char **itr;
+                  for (itr = built_modules; *itr != NULL; itr++)
+                    {
+                       snprintf(buf, sizeof(buf),
+                                "%s/src/modules/ecore/system/%s/.libs",
+                                PACKAGE_BUILD_DIR, *itr);
+                       module_list = eina_module_list_get(module_list, buf,
+                                                          EINA_FALSE, NULL, NULL);
+                    }
+
+                  if (module_list)
+                    eina_module_list_load(module_list);
+                  return;
                }
-
-             if (module_list)
-               eina_module_list_load(module_list);
-             return;
           }
      }
 
-   path = eina_module_environment_path_get("ECORE_MODULES_DIR",
-                                           "/ecore/system");
-   if (path)
-     {
-        module_list = eina_module_arch_list_get(module_list, path, MODULE_ARCH);
-        free(path);
-     }
-
-   path = eina_module_environment_path_get("HOME", "/.ecore/system");
-   if (path)
-     {
-        module_list = eina_module_arch_list_get(module_list, path, MODULE_ARCH);
-        free(path);
-     }
-
    snprintf(buf, sizeof(buf), "%s/ecore/system",
             eina_prefix_lib_get(_ecore_pfx));
    module_list = eina_module_arch_list_get(module_list, buf, MODULE_ARCH);
 
+   // XXX: MODFIX: do not list ALL modules and load them ALL! this is
+   // just polluting memory pages and I/O with modules and code that
+   // is then never used. detetc the module we need to use them use
+   // that. if anything load each module, have it do a detect and if
+   // it fails UNLOAD and try the next one.
    eina_module_list_load(module_list);
 }
 
@@ -616,9 +610,8 @@ ecore_main_loop_thread_safe_call_sync(Ecore_Data_Cb callback,
    order->sync = EINA_TRUE;
    order->suspend = EINA_FALSE;
 
-   _ecore_main_loop_thread_safe_call(order);
-
    eina_lock_take(&order->m);
+   _ecore_main_loop_thread_safe_call(order);
    eina_condition_wait(&order->c);
    eina_lock_release(&order->m);
 
@@ -1029,7 +1022,9 @@ _ecore_main_call_flush(void)
         else if (call->sync)
           {
              call->data = call->cb.sync(call->data);
+             eina_lock_take(&call->m);
              eina_condition_broadcast(&call->c);
+             eina_lock_release(&call->m);
           }
         else
           {
