@@ -192,7 +192,7 @@ eina_file_virtual_map_new(Eina_File *file,
         map->refcount = 0;
 
         eina_hash_add(file->map, &key, map);
-        eina_hash_direct_add(file->rmap, map->map, map);
+        eina_hash_direct_add(file->rmap, &map->map, map);
      }
 
    map->refcount++;
@@ -884,7 +884,7 @@ eina_file_copy(const char *src, const char *dst, Eina_File_Copy_Flags flags, Ein
      }
 
  end:
-   if (d > 0) close(d);
+   if (d >= 0) close(d);
    close(s);
 
    if (!success)
@@ -897,12 +897,15 @@ EAPI int
 eina_file_mkstemp(const char *templatename, Eina_Tmpstr **path)
 {
    char buffer[PATH_MAX];
-   const char *tmpdir;
+   const char *tmpdir = NULL;
    int fd;
    mode_t old_umask;
 
 #ifndef HAVE_EVIL
-   tmpdir = getenv("TMPDIR");
+#if defined(HAVE_GETUID) && defined(HAVE_GETEUID)
+   if (getuid() == geteuid())
+#endif
+     tmpdir = getenv("TMPDIR");
    if (!tmpdir) tmpdir = "/tmp";
 #else
    tmpdir = (char *)evil_tmpdir_get();
@@ -929,11 +932,14 @@ EAPI Eina_Bool
 eina_file_mkdtemp(const char *templatename, Eina_Tmpstr **path)
 {
    char buffer[PATH_MAX];
-   const char *tmpdir;
+   const char *tmpdir = NULL;
    char *tmpdirname;
 
 #ifndef HAVE_EVIL
-   tmpdir = getenv("TMPDIR");
+#if defined(HAVE_GETUID) && defined(HAVE_GETEUID)
+   if (getuid() == geteuid())
+#endif
+     tmpdir = getenv("TMPDIR");
    if (!tmpdir) tmpdir = "/tmp";
 #else
    tmpdir = (char *)evil_tmpdir_get();
