@@ -58,7 +58,7 @@ struct _Evas_Object_Textgrid
       Eina_Array                  palette_extended;
    } cur, prev;
 
-   int                            max_ascent;
+   int                            ascent;
 
    Evas_Font_Set                 *font;
 
@@ -297,14 +297,18 @@ evas_object_textgrid_textprop_ref(Evas_Object *eo_obj, Evas_Object_Textgrid *o,
           }
         offset--;
      }
-   if (o->master[offset].next[(codepoint & mask) >> shift] == 0)
+   if ((o->master[offset].next[(codepoint & mask) >> shift] == 0)
+       || ((o->master[offset].next[(codepoint & mask) >> shift] & 0xFFFFFF) >= o->glyphs_length))
      {
         Evas_Textgrid_Hash_Glyphs *tmp;
         unsigned char *tmp_used;
         int count;
 
         /* FIXME: find empty entry */
-        count = o->glyphs_length + 1;
+        if (o->master[offset].next[(codepoint & mask) >> shift] == 0)
+          count = o->glyphs_length + 1;
+        else
+          count = (o->master[offset].next[(codepoint & mask) >> shift] & 0xFFFFFF) + 1;
         tmp = realloc(o->glyphs, count * sizeof (Evas_Textgrid_Hash_Glyphs));
         if (!tmp) return 0xFFFFFFFF;
         o->glyphs = tmp;
@@ -312,10 +316,11 @@ evas_object_textgrid_textprop_ref(Evas_Object *eo_obj, Evas_Object_Textgrid *o,
         if (!tmp_used) return 0xFFFFFFFF;
         o->glyphs_used = tmp_used;
 
+        // FIXME: What should we write when allocating more than one new entry?
         o->master[offset].next[(codepoint & mask) >> shift] = o->glyphs_length + 0xFF000000;
 
-        memset(o->glyphs + o->glyphs_length, 0, sizeof (Evas_Textgrid_Hash_Glyphs));
-        o->glyphs_used[o->glyphs_length] = 0;
+        memset(o->glyphs + o->glyphs_length, 0, (count - o->glyphs_length) * sizeof (Evas_Textgrid_Hash_Glyphs));
+        memset(o->glyphs_used, 0, (count - o->glyphs_length) * sizeof(o->glyphs_used[0]));
         o->glyphs_length = count;
      }
 
@@ -664,11 +669,11 @@ evas_object_textgrid_render(Evas_Object *eo_obj,
                        // merged into horizontal runs like bg rects above
                        if (cells->underline)
                          evas_object_textgrid_row_line_append(row, xp, w,
-                                                              o->max_ascent + 1,
+                                                              o->ascent + 1,
                                                               c->r, c->g, c->b, c->a);
                        if (cells->strikethrough)
                          evas_object_textgrid_row_line_append(row, xp, w,
-                                                              ((3 * o->max_ascent) / 4),
+                                                              ((3 * o->ascent) / 4),
                                                               c->r, c->g, c->b, c->a);
                     }
                }
@@ -745,7 +750,7 @@ evas_object_textgrid_render(Evas_Object *eo_obj,
 
                   async_unref =
                     ENFN->multi_font_draw(output, context, surface,
-                                          o->font, xp, yp + o->max_ascent,
+                                          o->font, xp, yp + o->ascent,
                                           ww, hh, ww, hh, texts, do_async);
                   if (async_unref)
                     evas_unref_queue_texts_put(obj->layer->evas, texts);
@@ -764,7 +769,7 @@ evas_object_textgrid_render(Evas_Object *eo_obj,
                        Evas_Text_Props *props;
                        unsigned int     r, g, b, a;
                        int              tx = xp + row->texts[xx].x;
-                       int              ty = yp + o->max_ascent;
+                       int              ty = yp + o->ascent;
 
                        props =
                          evas_object_textgrid_textprop_int_to
@@ -910,13 +915,29 @@ evas_object_textgrid_render_pre(Evas_Object *eo_obj,
                {
                   Evas_Object_Textgrid_Row *r = &(o->cur.rows[i]);
                   if (r->ch1 >= 0)
-                    evas_add_rect(&obj->layer->evas->clip_changes,
-                                  obj->cur->geometry.x + 
-                                  (r->ch1 * o->cur.char_width),
-                                  obj->cur->geometry.y + 
-                                  (i * o->cur.char_height),
-                                  (r->ch2 - r->ch1 + 1) * o->cur.char_width,
-                                  o->cur.char_height);
+                    {
+                       Evas_Coord chx, chy, chw, chh;
+                       
+                       chx = r->ch1 * o->cur.char_width;
+                       chy = i * o->cur.char_height;
+                       chw = (r->ch2 - r->ch1 + 1) * o->cur.char_width;
+                       chh = o->cur.char_height;
+                       
+                       chx -= o->cur.char_width;
+                       chy -= o->cur.char_height;
+                       chw += o->cur.char_width * 2;
+                       chh += o->cur.char_height * 2;
+                       
+                       chx += obj->cur->geometry.x;
+                       chy += obj->cur->geometry.y;
+                       RECTS_CLIP_TO_RECT(chx, chy, chw, chh,
+                                          obj->cur->cache.clip.x,
+                                          obj->cur->cache.clip.y,
+                                          obj->cur->cache.clip.w,
+                                          obj->cur->cache.clip.h);
+                       evas_add_rect(&obj->layer->evas->clip_changes,
+                                     chx, chy, chw, chh);
+                    }
                }
           }
      }
@@ -1025,6 +1046,7 @@ evas_object_textgrid_scale_update(Evas_Object *eo_obj,
    o->cur.font_size = 0;
    o->prev.font_size = 0;
    evas_object_textgrid_font_set(eo_obj, font_name, font_size);
+   eina_stringshare_del(font_name);
 }
 
 /*********************  LOCAL *********************/
@@ -1251,7 +1273,7 @@ _font_set(Eo *eo_obj, void *_pd, va_list *list)
                                   obj->cur->scale));
    if (o->font)
      {
-        Eina_Unicode W[2] = { 'W', 0 };
+        Eina_Unicode W[2] = { 'O', 0 };
         Evas_Font_Instance *script_fi = NULL;
         Evas_Font_Instance *cur_fi = NULL;
         Evas_Text_Props text_props;
@@ -1266,14 +1288,11 @@ _font_set(Eo *eo_obj, void *_pd, va_list *list)
         ENFN->font_text_props_info_create(ENDT, script_fi, W, &text_props,
                                           NULL, 0, 1,
                                           EVAS_TEXT_PROPS_MODE_NONE);
-        ENFN->font_string_size_get(ENDT, o->font, &text_props,
-                                   &o->cur.char_width, &o->cur.char_height);
-        o->max_ascent = ENFN->font_max_ascent_get(ENDT, o->font);
-//        inset = ENFN->font_inset_get(ENDT, o->font, &text_props);
         advance = ENFN->font_h_advance_get(ENDT, o->font, &text_props);
         vadvance = ENFN->font_v_advance_get(ENDT, o->font, &text_props);
-        if (advance > o->cur.char_width) o->cur.char_width = advance;
-        if (vadvance > o->cur.char_height) o->cur.char_height = vadvance;
+        o->cur.char_width = advance;
+        o->cur.char_height = vadvance;
+        o->ascent = ENFN->font_ascent_get(ENDT, o->font);;
         evas_common_text_props_content_unref(&text_props);
      }
    else
@@ -1285,7 +1304,7 @@ _font_set(Eo *eo_obj, void *_pd, va_list *list)
           }
         EINA_COW_STATE_WRITE_END(obj, state_write, cur);
 
-        o->max_ascent = 0;
+        o->ascent = 0;
      }
 
    o->changed = 1;
