@@ -128,32 +128,42 @@ _socket_path_set(char *path)
    char *env;
    char buf[UNIX_PATH_MAX];
 
-   env = getenv("EVAS_CSERVE2_SOCKET");
-   if (env && env[0])
+#if defined(HAVE_GETUID) && defined(HAVE_GETEUID)
+   if (getuid() == geteuid())
+#endif
      {
-        eina_strlcpy(path, env, UNIX_PATH_MAX);
-        return;
+        env = getenv("EVAS_CSERVE2_SOCKET");
+        if (env && env[0])
+          {
+             eina_strlcpy(path, env, UNIX_PATH_MAX);
+             return;
+          }
      }
 
    snprintf(buf, sizeof(buf), "/tmp/.evas-cserve2-%x.socket", (int)getuid());
    /* FIXME: check we can actually create this socket */
    strcpy(path, buf);
 #if 0   
-   env = getenv("XDG_RUNTIME_DIR");
-   if (!env || !env[0])
+#if defined(HAVE_GETUID) && defined(HAVE_GETEUID)
+   if (getuid() == geteuid())
+#endif
      {
-        env = getenv("HOME");
+        env = getenv("XDG_RUNTIME_DIR");
         if (!env || !env[0])
           {
-             env = getenv("TMPDIR");
+             env = getenv("HOME");
              if (!env || !env[0])
-               env = "/tmp";
+               {
+                  env = getenv("TMPDIR");
+                  if (!env || !env[0])
+                  env = "/tmp";
+               }
           }
-     }
 
-   snprintf(buf, sizeof(buf), "%s/evas-cserve2-%x.socket", env, getuid());
-   /* FIXME: check we can actually create this socket */
-   strcpy(path, buf);
+        snprintf(buf, sizeof(buf), "%s/evas-cserve2-%x.socket", env, getuid());
+        /* FIXME: check we can actually create this socket */
+        strcpy(path, buf);
+     }
 #endif   
 }
 
@@ -418,7 +428,11 @@ _server_send(void *buf, int size, Op_Callback cb, void *data)
 
 on_error:
    if (!_request_answer_required(type, NULL))
-     return EINA_FALSE;
+     {
+        free(buf);
+        return EINA_FALSE;
+     }
+
    ERR("Socket error: %d %m", errno);
    switch (errno)
      {
@@ -507,7 +521,7 @@ evas_cserve2_shutdown(void)
 
    if (cserve2_init <= 0)
      {
-        CRIT("cserve2 is already shutdown");
+        CRI("cserve2 is already shutdown");
         return -1;
      }
 
@@ -590,7 +604,14 @@ _server_dispatch(Eina_Bool *failed)
 
    rid = msg->rid;
    if (!found)
-     WRN("Got unexpected response %d for request %d", msg->type, rid);
+     {
+        if (msg->type == CSERVE2_ERROR)
+          {
+             Msg_Error *error = (Msg_Error *) msg;
+             ERR("Cserve2 sent error %d for rid %d", error->error, rid);
+          }
+        else WRN("Got unexpected response %d for request %d", msg->type, rid);
+     }
 
    free(msg);
    return rid;
@@ -701,9 +722,11 @@ _image_opened_cb(void *data, const void *msg_received, int size)
           }
         else
           ERR("Invalid message type received: %d (%s)", answer->type, __FUNCTION__);
-        fentry = ie->data1;
         EINA_REFCOUNT_UNREF(fentry)
-          eina_hash_del(_file_entries, fentry->hkey, fentry);
+          {
+             eina_hash_del(_file_entries, fentry->hkey, fentry);
+             ie->data1 = NULL;
+          }
         return EINA_TRUE;
      }
 
@@ -929,6 +952,60 @@ _evas_image_load_opts_empty(Evas_Image_Load_Opts *lo)
            && (lo->scale_load.scale_hint == 0)); // Skip smooth
 }
 
+// Valgrind complains about uninitialized memory, because the load_opts
+// can be allocated on the stack, and each field is set independently,
+// leaving empty spaces in the struct non initialized. So, compare fields
+// one by one, don't take shortcuts like memcmp.
+
+static Eina_Bool
+_evas_image_load_opts_equal(const Evas_Image_Load_Opts *lo1,
+                            const Evas_Image_Load_Opts *lo2)
+{
+   return ((lo1->scale_down_by == lo2->scale_down_by)
+       && (lo1->dpi == lo2->dpi)
+       && (lo1->w == lo2->w)
+       && (lo1->h == lo2->h)
+       && (lo1->region.x == lo2->region.x)
+       && (lo1->region.y == lo2->region.y)
+       && (lo1->region.w == lo2->region.w)
+       && (lo1->region.h == lo2->region.h)
+       && (lo1->scale_load.src_x == lo2->scale_load.src_x)
+       && (lo1->scale_load.src_y == lo2->scale_load.src_y)
+       && (lo1->scale_load.src_w == lo2->scale_load.src_w)
+       && (lo1->scale_load.src_h == lo2->scale_load.src_h)
+       && (lo1->scale_load.dst_w == lo2->scale_load.dst_w)
+       && (lo1->scale_load.dst_h == lo2->scale_load.dst_h)
+       && (lo1->scale_load.smooth == lo2->scale_load.smooth)
+       && (lo1->scale_load.scale_hint == lo2->scale_load.scale_hint)
+       && (lo1->orientation == lo2->orientation)
+       && (lo1->degree == lo2->degree));
+}
+
+static void
+_evas_image_load_opts_set(Evas_Image_Load_Opts *lo1,
+                          const Evas_Image_Load_Opts *lo2)
+{
+   memset(lo1, 0, sizeof(Evas_Image_Load_Opts));
+   lo1->scale_down_by = lo2->scale_down_by;
+   lo1->dpi = lo2->dpi;
+   lo1->w = lo2->w;
+   lo1->h = lo2->h;
+   lo1->region.x = lo2->region.x;
+   lo1->region.y = lo2->region.y;
+   lo1->region.w = lo2->region.w;
+   lo1->region.h = lo2->region.h;
+   lo1->scale_load.src_x = lo2->scale_load.src_x;
+   lo1->scale_load.src_y = lo2->scale_load.src_y;
+   lo1->scale_load.src_w = lo2->scale_load.src_w;
+   lo1->scale_load.src_h = lo2->scale_load.src_h;
+   lo1->scale_load.dst_w = lo2->scale_load.dst_w;
+   lo1->scale_load.dst_h = lo2->scale_load.dst_h;
+   lo1->scale_load.smooth = lo2->scale_load.smooth;
+   lo1->scale_load.scale_hint = lo2->scale_load.scale_hint;
+   lo1->orientation = lo2->orientation;
+   lo1->degree = lo2->degree;
+}
+
 static void
 _file_hkey_get(char *buf, size_t sz, const char *path, const char *key,
                Evas_Image_Load_Opts *lo)
@@ -968,7 +1045,8 @@ _image_open_server_send(Image_Entry *ie)
    File_Entry *fentry;
    Data_Entry *dentry;
    const char *file, *key;
-   Evas_Image_Load_Opts *opts = NULL;
+   Evas_Image_Load_Opts opts;
+   Eina_Bool has_load_opts = EINA_FALSE;
 
    if (cserve2_init == 0)
      {
@@ -977,10 +1055,10 @@ _image_open_server_send(Image_Entry *ie)
      }
 
    if (!_evas_image_load_opts_empty(&ie->load_opts))
-     opts = &ie->load_opts;
-
-   ie->data1 = NULL;
-   ie->data2 = NULL;
+     {
+        _evas_image_load_opts_set(&opts, &ie->load_opts);
+        has_load_opts = EINA_TRUE;
+     }
 
    file = ie->file;
    key = ie->key;
@@ -998,7 +1076,7 @@ _image_open_server_send(Image_Entry *ie)
 
    hkey_len = flen + klen + 1024;
    hkey = alloca(hkey_len);
-   _file_hkey_get(hkey, hkey_len, filebuf, key, opts);
+   _file_hkey_get(hkey, hkey_len, filebuf, key, (has_load_opts ? &opts : NULL));
    fentry = eina_hash_find(_file_entries, hkey);
    if (!fentry)
      {
@@ -1014,47 +1092,53 @@ _image_open_server_send(Image_Entry *ie)
    else
      EINA_REFCOUNT_REF(fentry);
 
-   dentry = calloc(1, sizeof(*dentry));
-   if (!dentry)
+   if (!ie->data2)
      {
-        EINA_REFCOUNT_UNREF(fentry)
-          eina_hash_del(_file_entries, fentry->hkey, fentry);
-        return 0;
+        dentry = calloc(1, sizeof(*dentry));
+        if (!dentry)
+          {
+             EINA_REFCOUNT_UNREF(fentry)
+               eina_hash_del(_file_entries, fentry->hkey, fentry);
+             return 0;
+          }
+        dentry->image_id = ++_data_id;
      }
+   else dentry = ie->data2;
 
    memset(&msg_open, 0, sizeof(msg_open));
-
    msg_open.base.rid = _next_rid();
    msg_open.base.type = CSERVE2_OPEN;
    msg_open.file_id = fentry->file_id;
    msg_open.path_offset = 0;
    msg_open.key_offset = flen;
-   msg_open.has_load_opts = (opts != NULL);
-   msg_open.image_id = ++_data_id;
+   msg_open.has_load_opts = has_load_opts;
+   msg_open.image_id = dentry->image_id;
 
    size = sizeof(msg_open) + flen + klen;
-   if (opts)
-     size += sizeof(*opts);
+   if (has_load_opts)
+     size += sizeof(opts);
    buf = malloc(size);
    if (!buf)
      {
         EINA_REFCOUNT_UNREF(fentry)
           eina_hash_del(_file_entries, fentry->hkey, fentry);
         free(dentry);
+        ie->data2 = NULL;
         return 0;
      }
    memcpy(buf, &msg_open, sizeof(msg_open));
    memcpy(buf + sizeof(msg_open), filebuf, flen);
    memcpy(buf + sizeof(msg_open) + flen, key, klen);
-   if (opts)
-     memcpy(buf + sizeof(msg_open) + flen + klen, opts, sizeof(*opts));
+   if (has_load_opts)
+     memcpy(buf + sizeof(msg_open) + flen + klen, &opts, sizeof(opts));
 
    if (!_server_send(buf, size, _image_opened_cb, ie))
      {
         ERR("Couldn't send message to server.");
-        free(dentry);
         EINA_REFCOUNT_UNREF(fentry)
           eina_hash_del(_file_entries, fentry->hkey, fentry);
+        free(dentry);
+        ie->data2 = NULL;
         return 0;
      }
 
@@ -1717,7 +1801,7 @@ _glyph_map_remap_check(Glyph_Map *map, const char *idxpath, const char *datapath
         // Reopen mempool
         if (EINA_REFCOUNT_GET(&map->mempool) > 0)
           {
-             oldbuf = calloc(1, sizeof(Glyph_Map));
+             oldbuf = calloc(1, sizeof(*oldbuf));
              oldbuf->f = map->mempool.f;
              oldbuf->data = map->mempool.data;
              oldbuf->size = map->mempool.size;
@@ -1779,7 +1863,7 @@ _glyph_map_remap_check(Glyph_Map *map, const char *idxpath, const char *datapath
 
         if (EINA_REFCOUNT_GET(&map->mempool) > 0)
           {
-             oldbuf = calloc(1, sizeof(Glyph_Map));
+             oldbuf = calloc(1, sizeof(*oldbuf));
              oldbuf->f = eina_file_dup(map->mempool.f);
              oldbuf->data = map->mempool.data;
              oldbuf->size = map->mempool.size;
@@ -1882,10 +1966,11 @@ _font_entry_glyph_map_rebuild_check(Font_Entry *fe, Font_Hint_Flags hints)
              gl->base.bitmap.rows = gd->rows;
              gl->base.bitmap.width = gd->width;
              gl->base.bitmap.pitch = gd->pitch;
-             gl->base.bitmap.buffer = (unsigned char *)
-                   fe->map->mempool.data + gl->offset;
-             gl->base.bitmap.num_grays = gd->num_grays;
-             gl->base.bitmap.pixel_mode = gd->pixel_mode;
+             gl->base.bitmap.buffer = NULL;
+             gl->base.rle = (unsigned char *)
+               fe->map->mempool.data + gl->offset;
+             gl->base.rle_size = gl->size;
+             gl->base.bitmap.rle_alloc = EINA_FALSE;
              gl->idx = gd->index;
              gl->rid = 0;
 
@@ -1992,7 +2077,7 @@ _glyph_request_cb(void *data, const void *msg, int size)
      {
         string_t shm_id;
         unsigned int idx, offset, glsize, hints;
-        int rows, width, pitch, num_grays, pixel_mode;
+        int rows, width, pitch;
         CS_Glyph_Out *gl;
 
         pos = buf - (const char*) resp;
@@ -2013,10 +2098,6 @@ _glyph_request_cb(void *data, const void *msg, int size)
         buf += sizeof(int);
         memcpy(&pitch, buf, sizeof(int));
         buf += sizeof(int);
-        memcpy(&num_grays, buf, sizeof(int));
-        buf += sizeof(int);
-        memcpy(&pixel_mode, buf, sizeof(int));
-        buf += sizeof(int);
         memcpy(&hints, buf, sizeof(int));
         buf += sizeof(int);
         if (hints != grd->hints)
@@ -2042,10 +2123,12 @@ _glyph_request_cb(void *data, const void *msg, int size)
         gl->base.bitmap.rows = rows;
         gl->base.bitmap.width = width;
         gl->base.bitmap.pitch = pitch;
-        gl->base.bitmap.buffer =
+        gl->base.bitmap.buffer = NULL;
+        gl->base.bitmap.rle_alloc = 0;
+        gl->base.bitmap.no_free_glout = 1;
+        gl->base.rle =
               (unsigned char *) gl->map->mempool.data + gl->offset;
-        gl->base.bitmap.num_grays = num_grays;
-        gl->base.bitmap.pixel_mode = pixel_mode;
+        gl->base.rle_size = gl->size;
         gl->rid = 0;
 
         if (!eina_clist_element_is_linked(&gl->map_entry))
@@ -2515,8 +2598,8 @@ _server_index_list_set(Msg_Base *data, int size)
 
    if (size != sizeof(*msg) || msg->base.type != CSERVE2_INDEX_LIST)
      {
-        CRIT("Invalid message! type: %d, size: %d (expected %d)",
-             msg->base.type, size, (int) sizeof(*msg));
+        CRI("Invalid message! type: %d, size: %d (expected %d)",
+            msg->base.type, size, (int) sizeof(*msg));
         return -1;
      }
 
@@ -2582,7 +2665,7 @@ _shared_string_internal_get(int id, Eina_Bool safe)
 
    if (!_index.strings_entries.data)
      {
-        CRIT("Strings table is not valid: no data");
+        CRI("Strings table is not valid: no data");
         return NULL;
      }
 
@@ -2622,7 +2705,7 @@ _shared_string_get(int id)
 
 #define SHARED_INDEX_CHECK(si, typ) \
    do { if (!_shared_index_remap_check(&(si), sizeof(typ))) { \
-   CRIT("Failed to remap index"); return NULL; } } while (0)
+   CRI("Failed to remap index"); return NULL; } } while (0)
 
 static const File_Data *
 _shared_image_entry_file_data_find(Image_Entry *ie)
@@ -2644,12 +2727,17 @@ _shared_image_entry_file_data_find(Image_Entry *ie)
    if (!_index.files.header || !_index.files.entries.filedata)
      return NULL;
 
+   if (!ie->data1)
+     return NULL;
+
    // Direct access
    fe = ie->data1;
    if (fe->server_file_id)
      {
         if ((fdata = _shared_file_data_get_by_id(fe->server_file_id)) != NULL)
-          return fdata;
+          if (!fdata->changed)
+            return fdata;
+        fe->server_file_id = 0;
      }
 
    // Check hash
@@ -2671,6 +2759,7 @@ _shared_image_entry_file_data_find(Image_Entry *ie)
         fd = &(_index.files.entries.filedata[k]);
         if (!fd->id) break;
         if (!fd->refcount) continue;
+        if (fd->changed) continue;
 
         key = _shared_string_safe_get(fd->key);
         file = _shared_string_safe_get(fd->path);
@@ -2782,9 +2871,12 @@ _shared_file_data_get_by_id(unsigned int id)
 static inline Eina_Bool
 _shared_image_entry_image_data_match(Image_Entry *ie, const Image_Data *id)
 {
-   int cmp;
-   cmp = memcmp(&ie->load_opts, &id->opts, sizeof(ie->load_opts));
-   if (!cmp)
+   const Evas_Image_Load_Opts *lo1, *lo2;
+
+   lo1 = &ie->load_opts;
+   lo2 = &id->opts;
+
+   if (_evas_image_load_opts_equal(lo1, lo2))
      {
         DBG("Found loaded image entry at %d", id->id);
         return EINA_TRUE;
@@ -2947,7 +3039,6 @@ _shared_image_entry_image_data_find(Image_Entry *ie)
    Eina_Bool add_to_hash = SHARED_INDEX_ADD_TO_HASH;
    int k;
 
-
    DBG("Trying to find if image '%s:%s' is already loaded by cserve2",
        ie->file, ie->key);
 
@@ -2959,7 +3050,12 @@ _shared_image_entry_image_data_find(Image_Entry *ie)
 
    if (!ie->cache_key)
      {
-        CRIT("Looking for an image in remote cache without hash key?");
+        if (ie->flags.dirty)
+          {
+             DBG("Image is dirty (file changed?), not in cache.");
+             return NULL;
+          }
+        CRI("Looking for an image in remote cache without hash key?");
         return NULL;
      }
 
@@ -3013,6 +3109,7 @@ _shared_image_entry_image_data_find(Image_Entry *ie)
                   continue;
                }
 
+             if (fd->changed || !fd->valid) continue;
              key = _shared_string_safe_get(fd->key);
              file = _shared_string_safe_get(fd->path);
              if (!file)
